function struct2fitshead, hdrstr, $
	 use_sxaddpar=use_sxaddpar, use_fxaddpar, comments=comments
;+
;   Name: struct2fitshead
;
;   Purpose: Map FITS Header-Structure structure back-> fits header (strarr)
;
;   Input Parameters:
;      hdrstr - IDL structure to map into fits header (ex: output of mreadfits)
;
;   Output:
;      function returns FITS header derived from structure
;
;   Calling Sequence:
;      header=struct2fitshead(structure)
;
;   Calling Example:
;	mreadfits,f,hdr,data
;	ahdr = struct2fitshead(hdr(0))
;	writefits,"new-"+f(0),f(0),ahdr
;
;   History:
;      Craig DeForrest w/S.L.Freeland - 11-apr-1997
;      2-March-1998 - S.L.Freeland add some error checking...
;                     add /USE_SXADDPAR keyword (eventual default?)
;                     (default if NAXIS etc missing - just do it)
;      6-Mar-1998 - S.L.Freeland - add COMMENTS input keyword
;      6-May-1998 - S.L.Freeland - ORDER a few standard FITS
;     23-Jun-1998 - C.E. DeForest - call ID_UNESC before stuffing the tags
;		    into the fits header (complements ID_ESC in fitshead2struct).
;      3-Feb-2000 - S.L.Freeland - NOTE: made USE_SXADDPAR the default
;                                  (more forgiving about TAG contents/order)
;     14-Mar-2001 - S.M.Hill - corrected error in sorting code, needed trailing
;                              space on 'naxis' e.g. 'naxis '.
;
;   Calls:
;      fxaddpar, sxaddpar
;
;-
common struct2fitshead_blk, lastproc

if not data_chk(hdrstr,/struct) then begin
   box_message,['Structure required','IDL> header=struct2fitshead(structure)']
   return,''
endif

;  validity check (sxaddpar permits 'malformed' headers, ok in many applications
vindex=where(tag_index(hdrstr,str2arr('BITPIX,NAXIS,NAXIS1')) eq -1,ivcnt)
use_fxaddpar=keyword_set(use_fxaddpar)
use_sxaddpar=1-use_fxaddpar                      ; default='sxaddpar'
addproc=(['fxaddpar','sxaddpar'])(use_sxaddpar)  ; choose procedure

if n_elements(lastproc) eq 0 then lastproc=''
if lastproc ne addproc then begin
   box_message,'struct2fitshead - using procedure: ' + addproc
   lastproc=addproc
endif

if addproc eq 'sxaddpar' then ohdr=strarr(1)              ; diff initial val

tags = tag_names(hdrstr)
ntags=n_elements(tags)

coms=strarr(ntags)
if n_elements(comments) eq ntags then coms=comments       ; user supplied

for i=0,n_elements(tags)-1 do begin
   value=hdrstr.(i)
   if n_elements(value) gt 1 then begin                   ; comment&history
      nonnulls=where(strtrim(value,2) ne '',nncnt)
      for j=0,nncnt-1 do call_procedure,addproc,ohdr,id_unesc(tags(i)),(hdrstr.(i))(nonnulls(j))
   endif else begin
      value=hdrstr.(i)
      if data_chk(value,/type) eq 1 then value=fix(value)
      call_procedure, addproc, ohdr, id_unesc(tags(i)),value, coms(i)
   endelse
endfor

; minimal standard ordering

fitsstand=strupcase(str2arr('simple,bitpix,naxis ,naxis1,naxis2,naxis3'))
movepat='????'
for i=n_elements(fitsstand)-1,0, -1 do begin
  chk=where(strpos(ohdr,fitsstand(i)) ne -1, ccnt)
  if ccnt gt 0 then begin
     prelines=ohdr(chk)
     ohdr(chk)=movepat
     ohdr=[prelines,ohdr]
   endif
   ohdr=ohdr(where(ohdr ne movepat))
endfor

return,ohdr
end


